import re
from markdown.util import string_type

BLOCK_LEVEL_ELEMENTS = [
    'address', 'article', 'aside', 'blockqoute', 'br', 'canvas', 'dd', 'div',
    'dl', 'fieldset', 'figcaption', 'figure', 'footer', 'form', 'h[1-6]',
    'header', 'hr', 'li', 'main', 'nav', 'noscript', 'ol', 'output', 'p',
    'pre', 'section', 'table', 'tfoot', 'ul', 'video', 'remove'
]  # TODO: TO MAKE CONFIGURABLE


def is_block_level(html, block_level_elements):
    '''
    Checks if the root element (or first element) of the given
    html is a block level element.
    Args:
        html: A string of the html to check.
        block_level_elements: A list of strings which are the
        block level elements.
    Returns:
        True if the first element is a block level element.
    '''
    m = re.match(r'^\<\/?([^ >]+)', html)
    if m:
        tag = m.group(1)
        if tag[0] in ('!', '?', '@', '%'):
            return True
        if isinstance(tag, string_type):
            elements = '|'.join(block_level_elements)
            block_elements_re = re.compile('^({})$'.format(elements),
                                           re.IGNORECASE)
            return block_elements_re.match(tag)
        return False
    return False



import markdown
from unittest.mock import Mock

from verto.VertoExtension import VertoExtension
from verto.processors.VideoBlockProcessor import VideoBlockProcessor
from verto.processors.errors.NoSourceLinkError import NoSourceLinkError
from verto.processors.errors.NoVideoIdentifierError import NoVideoIdentifierError
from verto.processors.errors.UnsupportedVideoPlayerError import UnsupportedVideoPlayerError
from verto.tests.ProcessorTest import ProcessorTest


class VideoTest(ProcessorTest):
    '''The Video processor is similar to the a generic tag
    except that it requires custom validation for Video
    urls to extract identifiers and embedded properly.
    '''

    def __init__(self, *args, **kwargs):
        '''Sets up a processor name and info for accessing testing
        assets.
        '''
        ProcessorTest.__init__(self, *args, **kwargs)
        self.processor_name = 'video'
        self.ext = Mock()
        self.ext.jinja_templates = {
                self.processor_name: ProcessorTest.loadJinjaTemplate(self, self.processor_name),
                'video-youtube': ProcessorTest.loadJinjaTemplate(self, 'video-youtube'),
                'video-vimeo': ProcessorTest.loadJinjaTemplate(self, 'video-vimeo')
                }
        self.ext.processor_info = ProcessorTest.loadProcessorInfo(self)

    def test_contains_no_video(self):
        '''Tests that input that doesn't use the video processor tag is
        unchanged.
        '''
        test_string = self.read_test_file(self.processor_name, 'contains_no_video.md')
        blocks = self.to_blocks(test_string)

        self.assertFalse(all(VideoBlockProcessor(self.ext, self.md.parser).test(blocks, block) for block in blocks), msg='"{}"'.format(test_string))

        converted_test_string = markdown.markdown(test_string, extensions=[self.verto_extension])
        expected_file_string = self.read_test_file(self.processor_name, 'contains_no_video_expected.html', strip=True)
        self.assertEqual(converted_test_string, expected_file_string)

    def test_youtube_embed_link(self):
        '''Tests youtube embed links are properly embedded.
        '''
        test_string = self.read_test_file(self.processor_name, 'youtube_embed_link.md')
        blocks = self.to_blocks(test_string)

        self.assertListEqual([False, True, False, False, False, False, False, False], [VideoBlockProcessor(self.ext, self.md.parser).test(blocks, block) for block in blocks], msg='"{}"'.format(test_string))

        converted_test_string = markdown.markdown(test_string, extensions=[self.verto_extension])
        expected_file_string = self.read_test_file(self.processor_name, 'youtube_embed_link_expected.html', strip=True)

        self.assertEqual(converted_test_string, expected_file_string)

    def test_youtube_watch_link(self):
        '''Tests that youtube links are converted into embedded form.
        '''
        test_string = self.read_test_file(self.processor_name, 'youtube_watch_link.md')
        blocks = self.to_blocks(test_string)

        self.assertListEqual([False, False, False, False, False, True, False], [VideoBlockProcessor(self.ext, self.md.parser).test(blocks, block) for block in blocks], msg='"{}"'.format(test_string))

        converted_test_string = markdown.markdown(test_string, extensions=[self.verto_extension])
        expected_file_string = self.read_test_file(self.processor_name, 'youtube_watch_link_expected.html', strip=True)
        self.assertEqual(converted_test_string, expected_file_string)

    def test_youtube_be_link(self):
        '''Tests that short youtube links are embedded.
        '''
        test_string = self.read_test_file(self.processor_name, 'youtube_be_link.md')
        blocks = self.to_blocks(test_string)

        self.assertListEqual([False, True, False], [VideoBlockProcessor(self.ext, self.md.parser).test(blocks, block) for block in blocks], msg='"{}"'.format(test_string))

        converted_test_string = markdown.markdown(test_string, extensions=[self.verto_extension])
        expected_file_string = self.read_test_file(self.processor_name, 'youtube_be_link_expected.html', strip=True)
        self.assertEqual(converted_test_string, expected_file_string)

    def test_vimeo_link(self):
        '''Tests that vimeo links are converted into embedded form.
        '''
        test_string = self.read_test_file(self.processor_name, 'vimeo_link.md')
        blocks = self.to_blocks(test_string)

        self.assertListEqual([False, False, True, False, False, False], [VideoBlockProcessor(self.ext, self.md.parser).test(blocks, block) for block in blocks], msg='"{}"'.format(test_string))

        converted_test_string = markdown.markdown(test_string, extensions=[self.verto_extension])
        expected_file_string = self.read_test_file(self.processor_name, 'vimeo_link_expected.html', strip=True)
        self.assertEqual(converted_test_string, expected_file_string)

    def test_vimeo_player_link(self):
        '''Tests that the vimeo player links are embedded.
        '''
        test_string = self.read_test_file(self.processor_name, 'vimeo_player_link.md')
        blocks = self.to_blocks(test_string)

        self.assertListEqual([False, False, True, False, False, False], [VideoBlockProcessor(self.ext, self.md.parser).test(blocks, block) for block in blocks], msg='"{}"'.format(test_string))

        converted_test_string = markdown.markdown(test_string, extensions=[self.verto_extension])
        expected_file_string = self.read_test_file(self.processor_name, 'vimeo_player_link_expected.html', strip=True)
        self.assertEqual(converted_test_string, expected_file_string)

    def test_multiple_youtube_links(self):
        '''Tests output of multiple youtube links.
        '''
        test_string = self.read_test_file(self.processor_name, 'multiple_youtube_links.md')
        blocks = self.to_blocks(test_string)

        self.assertListEqual([False, False, True, False, True, False, True, False, True, False, True], [VideoBlockProcessor(self.ext, self.md.parser).test(blocks, block) for block in blocks], msg='"{}"'.format(test_string))

        converted_test_string = markdown.markdown(test_string, extensions=[self.verto_extension])
        expected_file_string = self.read_test_file(self.processor_name, 'multiple_youtube_links_expected.html', strip=True)
        self.assertEqual(converted_test_string, expected_file_string)

    def test_multiple_vimeo_links(self):
        '''Tests output of multiple vimeo links.
        '''
        test_string = self.read_test_file(self.processor_name, 'multiple_vimeo_links.md')
        blocks = self.to_blocks(test_string)

        self.assertListEqual([True, True, True], [VideoBlockProcessor(self.ext, self.md.parser).test(blocks, block) for block in blocks], msg='"{}"'.format(test_string))

        converted_test_string = markdown.markdown(test_string, extensions=[self.verto_extension])
        expected_file_string = self.read_test_file(self.processor_name, 'multiple_vimeo_links_expected.html', strip=True)
        self.assertEqual(converted_test_string, expected_file_string)

    def test_youtube_and_vimeo_links(self):
        '''Test that youtube links and vimeo links work together in the
        same document.
        '''
        test_string = self.read_test_file(self.processor_name, 'youtube_and_vimeo_links.md')
        blocks = self.to_blocks(test_string)

        self.assertListEqual([False, False, True, True, False, True, False, True, False, True, True, False, True, True], [VideoBlockProcessor(self.ext, self.md.parser).test(blocks, block) for block in blocks], msg='"{}"'.format(test_string))

        converted_test_string = markdown.markdown(test_string, extensions=[self.verto_extension])
        expected_file_string = self.read_test_file(self.processor_name, 'youtube_and_vimeo_links_expected.html', strip=True)
        self.assertEqual(converted_test_string, expected_file_string)

    def test_unsupported_video_type(self):
        '''Tests that links to other websites result in an
        UnsupportedVideoPlayerError exception.
        '''
        test_string = self.read_test_file(self.processor_name, 'unsupported_video_type.md')
        blocks = self.to_blocks(test_string)

        self.assertListEqual([False, False, True], [VideoBlockProcessor(self.ext, self.md.parser).test(blocks, block) for block in blocks], msg='"{}"'.format(test_string))

        self.assertRaises(UnsupportedVideoPlayerError, lambda x: markdown.markdown(x, extensions=[self.verto_extension]), test_string)

    def test_missing_identifier(self):
        '''Tests that a youtube link without an identifier will throw
        the NoVideoIdentifierError exception.
        '''
        test_string = self.read_test_file(self.processor_name, 'missing_identifier.md')
        blocks = self.to_blocks(test_string)

        self.assertListEqual([False, True, False], [VideoBlockProcessor(self.ext, self.md.parser).test(blocks, block) for block in blocks], msg='"{}"'.format(test_string))

        self.assertRaises(NoVideoIdentifierError, lambda x: markdown.markdown(x, extensions=[self.verto_extension]), test_string)

    #~
    # Doc Tests
    #~
    def test_doc_example_basic(self):
        '''A generic example of common usage.
        '''
        test_string = self.read_test_file(self.processor_name, 'doc_example_basic_usage.md')
        blocks = self.to_blocks(test_string)

        self.assertListEqual([True], [VideoBlockProcessor(self.ext, self.md.parser).test(blocks, block) for block in blocks], msg='"{}"'.format(test_string))

        converted_test_string = markdown.markdown(test_string, extensions=[self.verto_extension])
        expected_string = self.read_test_file(self.processor_name, 'doc_example_basic_usage_expected.html', strip=True)
        self.assertEqual(expected_string, converted_test_string)

    def test_doc_example_override_html(self):
        '''A example showing how to override the html template.
        '''
        test_string = self.read_test_file(self.processor_name, 'doc_example_override_html.md')
        blocks = self.to_blocks(test_string)

        self.assertListEqual([True], [VideoBlockProcessor(self.ext, self.md.parser).test(blocks, block) for block in blocks], msg='"{}"'.format(test_string))

        html_template = self.read_test_file(self.processor_name, 'doc_example_override_html_template.html', strip=True)
        verto_extension = VertoExtension([self.processor_name], html_templates={self.processor_name: html_template})

        converted_test_string = markdown.markdown(test_string, extensions=[verto_extension])
        expected_string = self.read_test_file(self.processor_name, 'doc_example_override_html_expected.html', strip=True)
        self.assertEqual(expected_string, converted_test_string)



from markdown.postprocessors import Postprocessor
from markdown.util import HtmlStash
from markdown.odict import OrderedDict
from bs4 import BeautifulSoup
import re


class BeautifyPostprocessor(Postprocessor):
    ''' Converts the output document into a more asthetically
    pleasing version with more consistent whitespace.
    '''

    def __init__(self, *args, **kwargs):
        ''' Creates a new BeautifyPostprocessor.
        '''
        super().__init__(*args, **kwargs)
        self.pre_pattern = re.compile(r'<pre>.*?</pre>', re.DOTALL)
        self.code_pattern = re.compile(r'<code>(?P<code>.*?)</code>', re.DOTALL)
        self.html_stash = HtmlStash()

    def run(self, text):
        '''Converts the document into a more asthetically
        pleasing version.
        Args:
            text: A string of the document to convert.
        Returns:
            A string of the converted document.
        '''
        text = self.store_non_pre_code_tags(text)
        text = BeautifulSoup(text, 'html.parser').prettify(formatter='html')
        text = self.restore_non_pre_code_tags(text)
        return text

    def store_non_pre_code_tags(self, text):
        '''Stores code tags that are not in pre tags to preserve
        whitespacing, replacing with placeholders.

        Args:
            text: A string of the document to process.
        Returns:
            The document with code blocks replaced with placeholders.
        '''
        prepass_text = text
        pre_spans = []

        match = self.pre_pattern.search(prepass_text)
        while match is not None:
            pre_spans.append(match.span())
            prepass_text = prepass_text[match.end():]
            match = self.pre_pattern.search(prepass_text)

        out_text = ''
        match = self.code_pattern.search(text)
        while match is not None:
            if not any(match.start() in range(*span) or match.end() in range(*span) for span in pre_spans):
                html_string = match.group()
                placeholder = self.html_stash.store(html_string, True)
                out_text = text[:match.start()] + placeholder
            else:
                out_text = text[:match.end()]
            text = text[match.end():]
            match = self.pre_pattern.search(text)
        out_text += text

        return out_text

    def restore_non_pre_code_tags(self, text):
        '''Restores code tags that are not in pre tags by replacing
         placeholders. This is to preserve whitespacing.

        Args:
            text: A string of the document to process.
        Returns:
            The document with placeholders replaced with stored
            code blocks.
        '''
        replacements = OrderedDict()
        for i in range(self.html_stash.html_counter):
            html, safe = self.html_stash.rawHtmlBlocks[i]
            replacements[self.html_stash.get_placeholder(i)] = html

        if len(replacements) > 0:
            pattern = re.compile('|'.join(re.escape(k) for k in replacements))
            text = pattern.sub(lambda m: replacements[m.group(0)], text)

        return text



import markdown
from unittest.mock import Mock

from verto.VertoExtension import VertoExtension
from verto.processors.CommentPreprocessor import CommentPreprocessor
from verto.tests.ProcessorTest import ProcessorTest

class BeautifyTest(ProcessorTest):
    '''The major concern with beautifying is that preformatted tags and
    code blocks are unchanged. The tests here cover these cases.
    '''

    def __init__(self, *args, **kwargs):
        '''Set processor name in class for file names.
        '''
        ProcessorTest.__init__(self, *args, **kwargs)
        self.processor_name = 'beautify'

    def test_example_inline_code(self):
        '''Tests to see that inline code formatting is unchanged.
        '''
        test_string = self.read_test_file(self.processor_name, 'example_inline_code.md')

        converted_test_string = markdown.markdown(test_string, extensions=[self.verto_extension])
        expected_string = self.read_test_file(self.processor_name, 'example_inline_code_expected.html', strip=True)
        self.assertEqual(expected_string, converted_test_string)

    def test_example_preformatted_code(self):
        '''Tests to ensure that preformatted tag content is unchanged.
        '''
        test_string = self.read_test_file(self.processor_name, 'example_preformatted_code.md')

        converted_test_string = markdown.markdown(test_string, extensions=[self.verto_extension])
        expected_string = self.read_test_file(self.processor_name, 'example_preformatted_code_expected.html', strip=True)
        self.assertEqual(expected_string, converted_test_string)

    def test_example_preformatted_code_with_extension(self):
        '''Tests to ensure that the fenced_code extension does not
        change output to retain compatibility.
        '''
        verto_extension = VertoExtension([self.processor_name], {}, ['markdown.extensions.fenced_code'])
        test_string = self.read_test_file(self.processor_name, 'example_preformatted_code_with_extension.md')

        converted_test_string = markdown.markdown(test_string, extensions=[verto_extension, 'markdown.extensions.fenced_code'])
        expected_string = self.read_test_file(self.processor_name, 'example_preformatted_code_with_extension_expected.html', strip=True)
        self.assertEqual(expected_string, converted_test_string)

    def text_example_mixed_code_types(self):
        '''Tests that all types of code blocks remain unchanged when
        used together.
        '''
        test_string = self.read_test_file(self.processor_name, 'example_mixed_code_types.md')

        converted_test_string = markdown.markdown(test_string, extensions=[self.verto_extension])
        expected_string = self.read_test_file(self.processor_name, 'example_mixed_code_types_expected.html', strip=True)
        self.assertEqual(expected_string, converted_test_string)
